package com.xs.core.filter;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotPermissionException;
import cn.dev33.satoken.jwt.exception.SaJwtException;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.xs.core.common.annotation.OperationLog;
import com.xs.core.common.exception.BusinessException;
import com.xs.core.model.ResponseResult;
import com.xs.core.service.log.ISysAppLogService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.ConstraintViolationException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.multipart.support.MissingServletRequestPartException;
import org.springframework.web.servlet.HandlerMapping;
import org.springframework.web.servlet.resource.NoResourceFoundException;

import java.lang.reflect.Method;
import java.util.*;
import java.util.function.Function;

@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {
    @Autowired
    private ISysAppLogService sysLogService;

    private final Map<Class<? extends Exception>, Function<Exception, ResponseResult<?>>> exceptionHandlers;

    public GlobalExceptionHandler() {
        exceptionHandlers = new HashMap<>();
        initializeHandlers();
    }

    private void initializeHandlers() {
        // 未登录或JWT异常
        exceptionHandlers.put(NotLoginException.class, e -> ResponseResult.unauthorized(null));
        exceptionHandlers.put(SaJwtException.class, e -> ResponseResult.unauthorized(null));

        // 请求格式相关异常
        exceptionHandlers.put(HttpMessageNotReadableException.class, e -> ResponseResult.badRequest(null));
        exceptionHandlers.put(HttpRequestMethodNotSupportedException.class, e -> ResponseResult.badRequest(null));
        exceptionHandlers.put(ConstraintViolationException.class, e -> ResponseResult.badRequest(e.getMessage()));

        // 文件上传异常
        exceptionHandlers.put(MaxUploadSizeExceededException.class, e -> ResponseResult.requestTooLarge(null));

        // 媒体类型异常
        exceptionHandlers.put(HttpMediaTypeNotSupportedException.class, e -> ResponseResult.unsupportedMediaType(null));

        // 权限异常
        exceptionHandlers.put(NotPermissionException.class, e -> ResponseResult.forbidden());

        // 业务异常
        exceptionHandlers.put(BusinessException.class, e -> ResponseResult.failed(e.getMessage()));

        // 文件缺失异常
        exceptionHandlers.put(MissingServletRequestPartException.class,
                e -> ResponseResult.failed("response.expected.file.missing", null));

        // 资源未找到异常
        exceptionHandlers.put(NoResourceFoundException.class,
                e -> ResponseResult.failed("request.api.not.exists", null));
    }


    @ExceptionHandler({Exception.class, NotLoginException.class, HttpMessageNotReadableException.class,
            HttpMediaTypeNotSupportedException.class, MaxUploadSizeExceededException.class, NotPermissionException.class,
            HttpRequestMethodNotSupportedException.class, ConstraintViolationException.class, MissingServletRequestPartException.class, SaJwtException.class, NoResourceFoundException.class, BusinessException.class})
    @ResponseBody
    ResponseResult<?> serverExceptionHandler(HttpServletRequest request, Exception e) {
        ResponseResult<?> result = exceptionHandlers.entrySet().stream()
                .filter(entry -> entry.getKey().isInstance(e))
                .findFirst()
                .map(entry -> entry.getValue().apply(e))
                .orElseGet(() -> {
                    log.error("请求:{} 异常,异常信息:{}", request.getRequestURI(), ExceptionUtil.stacktraceToString(e));
                    return ResponseResult.serverError(null);
                });
        saveExceptionLog(request, e, result);
        return result;
    }

    private void saveExceptionLog(HttpServletRequest request, Exception ex, ResponseResult<?> result) {
        // 获取方法上的日志注解
        try {
            HandlerMethod handlerMethod = (HandlerMethod) request.getAttribute(
                    HandlerMapping.BEST_MATCHING_HANDLER_ATTRIBUTE);
            OperationLog operationLog = handlerMethod.getMethodAnnotation(OperationLog.class);
            Method method = handlerMethod.getMethod();
            String methodName = method.getDeclaringClass().getName() + "." + method.getName();
            // 记录日志
            if (operationLog != null) {
                sysLogService.saveAppLog(
                        operationLog.description(),
                        operationLog.operationType(),
                        operationLog.module(),
                        methodName,
                        getMethodParams(request),
                        result,
                        0,
                        ex
                );
            }
        } catch (Exception e) {
            log.error("记录日志异常,异常信息:{}", ExceptionUtil.getMessage(e));
        }
    }

    public static Object getMethodParams(HttpServletRequest request) {
        try {
            // 获取请求方式
            String method = request.getMethod();
            Map<String, Object> paramsMap = new LinkedHashMap<>();

            // GET 请求从 URL 参数获取
            if ("GET".equals(method)) {
                Map<String, String[]> parameterMap = request.getParameterMap();
                for (Map.Entry<String, String[]> entry : parameterMap.entrySet()) {
                    paramsMap.put(entry.getKey(),
                            entry.getValue().length == 1 ? entry.getValue()[0] : Arrays.asList(entry.getValue()));
                }
            }
            // POST 请求从请求体获取
            else if ("POST".equals(method)) {
                // 获取 Content-Type
                String contentType = request.getContentType();
                if (contentType != null && contentType.contains("application/json")) {
                    // JSON 请求体
                    String body = (String) request.getAttribute("decryptRequestParams");
                    if (StrUtil.isNotBlank(body)) {
                        paramsMap = JSON.parseObject(body);
                    }
                } else {
                    // 表单参数
                    Map<String, String[]> parameterMap = request.getParameterMap();
                    for (Map.Entry<String, String[]> entry : parameterMap.entrySet()) {
                        paramsMap.put(entry.getKey(),
                                entry.getValue().length == 1 ? entry.getValue()[0] : Arrays.asList(entry.getValue()));
                    }
                }
            }
            return paramsMap;
        } catch (Exception e) {
            log.warn("获取方法参数失败", e);
            return "{}";
        }
    }


    @ExceptionHandler({MethodArgumentNotValidException.class, BindException.class})
    public ResponseResult<?> handleValidationException(Exception ex, HttpServletRequest request) {
        // 获取方法上的日志注解
        HandlerMethod handlerMethod = (HandlerMethod) request.getAttribute(
                HandlerMapping.BEST_MATCHING_HANDLER_ATTRIBUTE);
        OperationLog operationLog = handlerMethod.getMethodAnnotation(OperationLog.class);

        Method method = handlerMethod.getMethod();
        String methodName = method.getDeclaringClass().getName() + "." + method.getName();

        ResponseResult<Object> failed = ResponseResult.failed("response.validation.failed", null);
        List<FieldError> fieldErrorList = ((BindException) ex).getFieldErrors();
        if (!CollectionUtils.isEmpty(fieldErrorList)) {
            for (FieldError fieldError : fieldErrorList) {
                if (fieldError != null && fieldError.getDefaultMessage() != null) {
                    String defaultMessage = fieldError.getDefaultMessage();
                    // 尝试从消息源获取国际化消息，如果没有找到，则使用默认消息
                    failed = ResponseResult.failed(defaultMessage, null);
                    break;
                }
            }
        }

        // 记录日志
        if (operationLog != null) {
            sysLogService.saveAppLog(
                    operationLog.description(),
                    operationLog.operationType(),
                    operationLog.module(),
                    methodName,
                    request.getParameterMap(),
                    failed,
                    0,
                    ex
            );
        }
        // 返回错误信息
        return failed;
    }
}
